package windowsPackage;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Observable;

import javax.imageio.ImageIO;
import javax.swing.*;

public class SeatOverview extends Observable {

	private static SeatOverview instance;
	private JFrame frame;
	private Container contentPane;
	private int numberOfColumns = 0;
	private int numberOfRows = 0;
	private int travelNumber;
	private int flightNumber;
	private Seat[][] seats;
	private JPanel panel;
	private ArrayList<Integer> bookedSeats;
	private Database db;

	public SeatOverview(Booking bookingFrame, Integer flightNumber, Integer travelNumber, ArrayList<Integer> bookedSeats) throws SQLException {
		
		db = new Database();
		this.bookedSeats = bookedSeats;
		// Benytter quickSort til at sortere arrayListen med de valgte sder
		sortBookedSeats();
		this.travelNumber = travelNumber;
		this.flightNumber = flightNumber;
		frame = new JFrame("Seat Overview");
		contentPane = frame.getContentPane();
		panel = new JPanel();
		this.addObserver(bookingFrame);
		makeContent();
		frame.pack();
		frame.setVisible(true);
	}
	// Quicksort
	private void qsort(ArrayList<Integer> arr, int a, int b) {
		if(a < b) {
			int i = a, j = b;
			//Vlger en tal midt i arrayet
			int x = arr.get((i + j) / 2);
			do {
				//S lnge arr.get(i) (dvs. begyndelsen af arrayListen) er under det midterste tal, gr den videre til nste tal i arraylisten
				while (arr.get(i) < x) i++;
				//Samme som ovenfor, blot fra slutningen af araylisten og hen mod midten (x), og hvor arr.get(j) skal vre strre end x
				while (arr.get(j) > x) j--;
				// Hvis i er under eller lig j, m while lykkerne have fundet 2 vrdier, hvor den ene er strre end x, og den anden er mindre end x, og disse skal derfor byttes om.
				if(i <= j) {
					swap(arr, i, j);
					i++; j--;
				}
				// Forstter s lnge i <= j
			} while (i <= j);
			//Deler arrayey op i 2 dele og sorterer disse hver for sig.
			qsort(arr, a, j);
			qsort(arr, i, b);
		}
	}

	private void sortBookedSeats() {
		qsort(bookedSeats, 0, bookedSeats.size() - 1);
	}
	
	//Bytter 2 vrdier om i arrayListen
	private void swap(ArrayList<Integer> arr, int i, int j) {
		Integer tmp = arr.get(i);
		arr.set(i, arr.get(j));
		arr.set(j, tmp);
	}

	//Henter flyets detaljer ssom antal rkker og kolonner.
	private void setUp() {

		// stter instance til null nr vinduet lukkes, sledes at man kan bne vinduet igen pga. singleton
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				instance = null;
			}
		});

		try {
			String cmd = "Select `AirPlaneTypeID` FROM `AirPlanes` WHERE AirPlaneID = '" + flightNumber + "'";

			ResultSet rs = db.execute(cmd);
			int flightType = 0;
				rs.next();
				flightType = rs.getInt("AirPlaneTypeID");

				cmd = "Select * FROM `AirPlaneTypes` WHERE AirPlaneTypeID = '" + flightType + "'";
				rs = db.execute(cmd);

				rs.next();
				numberOfRows = rs.getInt("numberOfRows");
				numberOfColumns = rs.getInt("numberOfColumns");
			
		}
		catch(SQLException e) {
			JOptionPane.showMessageDialog(null, "Der skete en fejl!! \n Fejlkode: " + e);
		}
	}
	//Danner indholdet p siden.
	private void makeContent() {
		setUp();

		panel.setLayout(new GridLayout(2, 1, 0, 20));

		ImageIcon imgIconFront = null;
		ImageIcon imgIconEnd = null;
		URL url = null;

		try {
			url = new File("./src/windowsPackage/flyFront.png").toURI().toURL();
			BufferedImage bufferimg = ImageIO.read(url);
			Image img = bufferimg.getScaledInstance(200, 150,  java.awt.Image.SCALE_SMOOTH);  
			imgIconFront = new ImageIcon(img);

			url = new File("./src/windowsPackage/flyEnd.png").toURI().toURL();
			bufferimg = ImageIO.read(url);
			img = bufferimg.getScaledInstance(200, 150,  java.awt.Image.SCALE_SMOOTH);  
			imgIconEnd = new ImageIcon(img);


		} catch (IOException e) {
			System.out.println("Error, could not read image " + e + " URL: " + url);
		}

		JPanel planePanel = new JPanel();
		planePanel.setLayout(new FlowLayout());

		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new FlowLayout());

		JLabel imgLabelFront = new JLabel(imgIconFront);
		JLabel imgLabelEnd = new JLabel(imgIconEnd);

		JButton selectSeats = new JButton("Vlg Sder");
		selectSeats.addActionListener(new selectSeatsActionListener());

		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new cancelActionListener());

		planePanel.add(imgLabelFront);
		
		try {
			planePanel.add(getSeats());
		}
		catch(SQLException e) {
			JOptionPane.showMessageDialog(null, "Der skete en fejl! \n Fejlkode: " + e);
		}
		planePanel.add(imgLabelEnd);


		buttonsPanel.add(selectSeats);
		buttonsPanel.add(cancel);

		panel.add(planePanel);
		panel.add(buttonsPanel);
		contentPane.add(panel);

	}
	
	//Stter sderne fra databasen ind i et gridlayout, og indstter dem i et todimentionelt array.
	private JPanel getSeats() throws SQLException {
		ResultSet rs = getSeatsFromDB();

		JPanel seatsPanel = new JPanel();
		seatsPanel.setLayout(new GridLayout(numberOfColumns, numberOfRows, 10, 10));
		seats = new Seat[numberOfRows][numberOfColumns];	

			//Danner alle sderne fra ResultSettet
			Seat seat;
			int i = 0;
			while(rs.next()) {
				Boolean selected = false;
				Integer seatID = rs.getInt("SeatsID");
				if(i < bookedSeats.size()) {
					if(seatID.equals(bookedSeats.get(i))) {
						selected = true;
						i++;
					}
				}
				seat = new Seat(rs.getInt("SeatsID"), rs.getBoolean("Booked"), selected);
				
				
				seatsPanel.add(seats[rs.getInt("Row")-1][rs.getInt("Column")-1] = seat);
			}

		return seatsPanel;
	}

	private ResultSet getSeatsFromDB() throws SQLException {
			String cmd = "SELECT * FROM Seats WHERE TravelID = " + travelNumber;
			return db.execute(cmd);
	}

	//Singleton
	public static SeatOverview getInstance(Booking bookingFrame, Integer flightNumber, Integer travelNumber, ArrayList<Integer> bookedSeats) {
		if(instance == null)
			try {
				instance = new SeatOverview(bookingFrame, flightNumber, travelNumber, bookedSeats);
			}
		catch(SQLException e) {
			JOptionPane.showMessageDialog(null, "Du skal vre forbundet til internettet!");
			instance = null;
		}
		return instance;
	}

	private class selectSeatsActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			setChanged();
			// Opdaterer listen med hvilke sder der er valgt
			bookedSeats.removeAll(bookedSeats);
			for(int a = 0; a < numberOfRows; a++) { 
				for(int b = 0; b < numberOfColumns; b++) {
					if(seats[a][b].isSelected()) {
						bookedSeats.add(seats[a][b].getID());
					}
				}
			}
			notifyObservers(bookedSeats);
			instance = null;
			frame.dispose();
		}
	}

	private class cancelActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			instance = null;
			frame.dispose();
		}
	}

}
